use clap::Parser;
use clap::Subcommand;

use crate::types::Date;

pub mod dispatch;

pub use dispatch::dispatch;

/// Dead simple event monitor for dashboards.
#[derive(Parser, Debug)]
#[command(version, about, long_about = None, arg_required_else_help = true)]
pub struct Args {
    /// Config file location
    #[arg(short, long)]
    pub config_file: Option<String>,

    ///Log level.
    ///
    ///0: No Logging (Silent mode)
    ///1: Info
    ///2: Debug
    ///3: Trace
    #[arg(short, long, default_value_t = 0)]
    pub verbosity: u8,

    #[command(subcommand)]
    pub command: Command,
}

// TODO: Command line stuff! Interesting
#[derive(Subcommand, Debug, Clone)]
pub enum Command {
    /// Run webserver and collector
    Run,
    /// Manage API keys
    Api {
        #[command(subcommand)]
        subcommand: ApiCommand,
    },
    /// Print config file
    ShowConf,
    /// Delete dataset <name>.
    Delete {
        /// Name of the dataset
        name: String,
        /// Initial date
        from: Option<Date>,
        /// End date
        to: Option<Date>,
        /// Don't ask for confirmation
        #[arg(short, long, default_value_t = false)]
        force_yes: bool,
    },
    /// Query datasets
    ShowData,
    /// Show information and statistics
    Info,
}

#[derive(Subcommand, Debug, Clone)]
pub enum ApiCommand {
    /// Generates and prins a new api key
    Generate {
        /// Optional comment
        comment: Option<String>,
    },
    /// Shows all API keys
    List,
    /// Remove/revoke an API key.
    Delete {
        /// ID of the api key to remove. Use `dasher api list` to get the id.
        id: i64,
    },
}
