mod api;
use std::{
    io::{self, Write},
    process::exit,
};

use api::run_web_api;

mod tcp;
use tcp::listen_api;
use tokio::task;

mod cli;
mod conf;
mod db;
mod types;
use db::DB;
use tracing::info;
use tracing_subscriber::EnvFilter;

use types::{DATE_MAX, DATE_MIN};

use clap::Parser;

pub fn setup_logging(level: u8) {
    let log_level = match level {
        0 => return,
        1 => "info",
        2 => "debug",
        _ => "trace",
    };

    let filter = EnvFilter::new(format!("none,dasher={}", log_level));

    tracing_subscriber::fmt().with_env_filter(filter).init();
}

#[tokio::main(flavor = "current_thread")]
async fn main() {
    let args = cli::Args::parse();

    setup_logging(args.verbosity);

    let conf = {
        if let Some(path) = args.config_file.clone() {
            match conf::Config::new(&path).await {
                Ok(c) => c,
                Err(e) => {
                    info!("Config error: \n{:?}", e);
                    exit(1);
                }
            }
        } else {
            info!("No config found. Using default config. Specify configuration with -c option.");
            conf::Config::default()
        }
    };

    cli::dispatch(args, &conf).await;
}
